/*
* MIDICntrllr, December 2019.
* Author: José Borja Castillo Sánchez.
* Based on Alejandro Fernández's work.
*/
#include <MIDIUSB.h>
#include "MIDI_inputs.h"
#include "MIDI_timers.h"
#include "MIDI_operations.h"
#include <stdint.h>
#include <Wire.h>

#define KEYS_MUX_ADDR 0x20
#define GP0_ACCESS 0x00
#define GP1_ACCESS 0x01

#define MAX_TIME  10

volatile bool I2CPermission;
volatile uint16_t timeElapsed;

volatile uint8_t poly_notes[8];
volatile uint8_t poly_count;
volatile uint8_t hold_held;

volatile uint8_t note, volume, instrument, octave;

void keyRequest(uint8_t *v1, uint8_t *v2)
{
    Wire.beginTransmission(KEYS_MUX_ADDR);
    Wire.write(GP0_ACCESS);
    Wire.write(GP1_ACCESS);
    
    Wire.endTransmission();
    Wire.requestFrom(KEYS_MUX_ADDR, 2);    
    
    *v1 = Wire.read();
    *v2 = Wire.read();
}

void KEY_INT_ISR(void)
{
   //enable timer
   Timer3_Start();
}

void setup(void)
{
  I2CPermission = false;
  note = volume = 0;
  poly_count = 0;
  instrument = 60;
  timeElapsed = 0;
  hold_held = 0;
  Timers_Setup();
	// I2C (or TWI/Wire) uses pins 2 (SDA) and 3 (SCL).
	Wire.begin();
  pinMode(KEY_INT_PIN,INPUT);
	attachInterrupt(digitalPinToInterrupt(KEY_INT_PIN),KEY_INT_ISR,FALLING);
  interrupts();
}

void loop(void)
{
	//loop forever and ever...
  if(I2CPermission)  
  { 
    I2CPermission = false;
    uint8_t v1,v2;
    keyRequest(&v1,&v2);
    hold_held = decode_hold(v2);
    
    //send note off last note
    if (poly_count > 0)
    {
      if( (!hold_held))
      {
        // hold not held
        while(poly_count > 0)
        {
          MIDI_send_noteOFF(0,poly_notes[poly_count],0);
          poly_count--;
        }
        MIDI_send_noteOFF(0,poly_notes[0],0);
      }
    }
    else 
      MIDI_send_noteOFF(0,note,0);
    
    uint16_t v = v1 | (v2 << 8);
//    Serial.println(v,HEX);
    note = decode_note(&v);
    if(note < (sizeof(notes_addr) / sizeof(uint16_t)) )
    {
      //send current instrument
      MIDI_send_instrument(0,instrument);
      //send note on
      note += BASE_NOTE + OCTAVE_LEN*octave;
      if(hold_held)
      {
        //hold held
        poly_notes[poly_count] = note;
        poly_count++;
      }
      
      MIDI_send_noteON(0, note, 127);
      
      // send through I2C buffer new note.
    }

   }
   
   if((timeElapsed % MAX_TIME) == 0)
   {  
      timeElapsed = 0; 
      volume = decode_volume(AMPLITUDE_PIN);
      if(octave != decode_octave(OCTAVE_UPSHIFT_PIN))
      {
        octave = decode_octave(OCTAVE_UPSHIFT_PIN);
        I2CPermission = true;
      }
      MIDI_send_volume(0,volume);
   }
   timeElapsed = millis();
}
